/*
 * ICC Examin ist eine ICC Profil Betrachter
 * 
 * Copyright (C) 2004-2011  Kai-Uwe Behrmann 
 *
 * Autor: Kai-Uwe Behrmann <ku.b@gmx.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * -----------------------------------------------------------------------------
 *
 * content for the informative "About" window.
 * 
 */


#include "icc_utils.h"
#include "icc_examin_version.h"
#include "icc_helfer.h"
#include "config.log.h"
#include <sstream>


#define DEBUG_ICCUEBER

#define  SF "#cccccc"  // standard background colour

# define LAYOUTFARBE if (layout[l++] == true) \
                        html << HF; \
                      else \
                        html << SF; // select colour according to layout options

std::string
getHtmlKopf(const char* titel)
{
  std::stringstream html;
  html << "<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">"
       << "<meta http-equiv=\"Content-Type\" content=\"text/html\">\n"
       << "<html>\n<head>\n"
       << "<title>" << titel << "</title>\n"
       << "<meta http-equiv=\"content-type\" content=\"text/html; charset=ISO-8859-1\">\n"
       << "<meta name=\"description\" content=\"icc_examin ICC Profil Betrachter\">\n"
       << "<meta name=\"author\" content=\"automatic generated by icc_examin-" << ICC_EXAMIN_V << "\">\n"
       << "</head>\n<body bgcolor=\"" << SF << "\" text=\"#000000\" link=\"blue\">\n\n";
  return html.str();
}


std::string
getUeberHtml()
{
  DBG_PROG_START
  //char HF[] = "#aaaaaa";  // emphasised
  std::stringstream html;

  // HTML
  html << getHtmlKopf( _("About ICC Examin") );

  // general informations
  html <<
"<table cellspacing=3 cellpadding=4 width=\"100%\"> \n\
  <tr> \n\
    <td bgcolor=\"" << SF << "\"> \n\
    <center> \n\
    <h2>"
       // HTML
       << "  " << _("ICC Examin Version") << " " << ICC_EXAMIN_V  << " \
    </h2> \n\
    </center><br><br><br>. \n\
    </td> \n\
  </tr> \n\
  <tr> \n\
    <td bgcolor=\"" << "#9f9f9f" << "\"> \n\
    <b>Copyright &copy; 2004-"; html << DEVEL_YEAR <<"  Kai-Uwe Behrmann \n\
    </b> \n\
  </tr> \n\
</table> \n\
<br>";
  // HTML
  html << _("Date") << ": " << 
          START_MONTH<<" "<<START_YEAR<<" - "<<DEVEL_MONTH<<" "<<DEVEL_YEAR;
  html << " \n\
<br> \n\
<p>";
  // HTML
  html << _(
"internet: www.oyranos.org/icc-examin <br>");
  html << " \
</p>\n \
<p>";
  // HTML
  html << _(
"ICC Examin is a viewer for colour profiles according to ICC "
"standard  - www.color.org . At the moment version 2 and version 4 profiles are supported.");
  html << " \
</p>\n \
<p>";
  // HTML
  html << _(
"The appliance of color profiles shall help to achive an easy color data "
"exchange, regardless which application or what kind of operating system is "
"used. As well color adaption between different devices is possible, "
"provided the color characteristics of both devices are known. "
"The qulity of a color transform from one to an other device depends mainly "
"on the quality of the color measurement and the used profiling algorithm "
"during profile creation.");
  html << " \
</p>\n \
<p>";
  // HTML
  html << _(
"Color profiles are often available from manufacturers of optical devices "
"(digital camera, scanner, printer). With ICC Examin you can get an impression "
"of the internal data , stored in a ICC profile. For profiles containing "
"measurement data, ICC Examin can generate a quality report. The deviation, "
"calculated from the measurement data is shown as well in the three "
"dimensional gamut viewer.<br>");
  html << " \
</p>\n\
<p>";
  // HTML
  html << _(
"The color transformations are realised with littleCMS through the Oyranos "
"CMM API.");
  html << " \
</p>\n";

  //DBG_NUM_S(html.str() )
  DBG_PROG_ENDE
  return html.str();
}

std::string
getHilfeHtml()
{
  DBG_PROG_START
  std::stringstream html;
  // HTML
  html << getHtmlKopf( _("Help") );

  html << " <p>";
  html << _("ICC Examin is a colour profile viewer.");
  html << "</p><p>";
  html << _("It shows the internals of binary encoded ICC profiles to get an idea about its capabilities.");
  html << "</p> \
<table border=\"0\" id=\"toc\"><tr id=\"toctitle\"><td align=\"center\"> \
<b>" << _("Table of contents") << "</b></td></tr><tr id=\"tocinside\"><td> \
<div class=\"tocline\"><a href=\"#Features\">";
  html << _("1 Features"); html << "</a><br /></div> \
<div class=\"tocline\"><a href=\"#Usage\">";
  html << _("2 Usage"); html << "</a><br /></div> \
 \
<div class=\"tocindent\"> \
<p><a href=\"#File_opening\">";
  html << _("2.1 File opening"); html << "</a><br /> \
<a href=\"#Main_window\">";
  html << _("2.2 Main window"); html << "</a><br /> \
<a href=\"#Complex_tags_navigation\">";
  html << _("2.3 Complex tags navigation"); html << "</a><br /> \
<a href=\"#Windows_and_Views\">";
  html << _("2.4 Windows and Views"); html <<"</a> \
</p> \
<div class=\"tocindent\"> \
<p><a href=\"#Gamut\">";
  html << _("2.4.1 Gamut"); html << "</a><br /> \
<a href=\"#3D_navigation\">";
  html << _("2.4.2 3D navigation and keys"); html << "</a><br /> \
<a href=\"#Report_View\">";
  html << _("2.4.3 Report View"); html << "</a><br /> \
 \
<a href=\"#CGATS_view\">";
  html << _("2.4.4 CGATS view"); html << "</a><br /> \
<a href=\"#Grafic_card_gamma_tables\">";
  html << _("2.4.5 Grafic card gamma tables"); html << "</a><br /> \
<a href=\"#Help\">";
  html << _("2.4.6 Help"); html << "</a><br /> \
</p> \
</div> \
</div> \
<br/><div class=\"tocline\"><a href=\"#Profile_elements\">";
  html << _("3 Profile elements"); html << "</a><br /></div> \
<div class=\"tocindent\"> \
<p><a href=\"#Header\">";
  html << _("3.1 Header"); html << "</a><br /> \
<a href=\"#Tags\">";
  html << _("3.2 Tag types"); html << "</a><br /> \
</p> \
 \
<div class=\"tocindent\"> \
<p><a href=\"#text_.2F_desc_.2F_targ\">";
  html << _("3.2.1 text / desc / targ / mluc"); html << "</a><br /> \
<a href=\"#curv\">";
  html << _("3.2.2 curv"); html << "</a><br /> \
<a href=\"#XYZ\">";
  html << _("3.2.3 XYZ"); html << "</a><br /> \
<a href=\"#mft1_.2F_mft2\">";
  html << _("3.2.4 mft1 / mft2"); html << "</a><br /> \
<a href=\"#sf32\">";
  html << _("3.2.5 sf32"); html << "</a><br /> \
<a href=\"#ncl2\">";
  html << _("3.2.6 ncl2"); html << "</a><br /> \
<a href=\"#psid\">";
  html << _("3.2.7 psid"); html << "</a><br /> \
</p> \
</div> \
</div> \
<div class=\"tocline\"><a href=\"#Other_file_formats\">";
  html << _("4 Other file formats"); html << "</a><br /></div> \
 \
<div class=\"tocindent\"> \
<p><a href=\"#CGATS\">";
  html << _("4.1 Measurements CGATS"); html << "</a><br /> \
<a href=\"#VRML\">";
  html << _("4.2 Model VRML"); html << "</a><br /> \
</p> \
</div> \
</td></tr></table> \
<a name=\"Features\"></a><h2>";
  html << _("1 Features"); html << "</h2> \
<ul><li> "; html << _("For an overview see"); 
  html << " <a href=\"http://www.oyranos.org/icc-examin/\" class='external' title=\"http://www.oyranos.org/icc-examin/\">ICC Examin</a><span class='urlexpansion'>&nbsp;(<i>http://www.oyranos.org</i>)</span> \
</li></ul> \
<a name=\"Usage\"></a><h2>";
  html << _("2 Usage"); html << "</h2> \
<p>";
  html << _("ICC Examin can be launched from the Application folder or Dock on osX and from the Graphics menu on Linux. It should as well register as viewer for ICC profiles, CGATS text and VRML Gamut files.");
  html << "</p> \
<a name=\"File_opening\"></a><h3>";
  html << _("2.1 File opening"); html << "</h3> \
<p>";
  html << _("From the application windows menubar you can select File-&gt;Open and to show file dialog. Select one or more files for gamut comparision as you like. The selection order should be preserved. On top right side of the FLTK file selector resides the bookmark alias Favourites button. If you frequently visit directories you will find it usefull to append these directories to your bookmark / favourites list. To do this, go to an directory with the file selector and choose from the bookmark button the add entry. The added directory appears in the bookmark list and is available for later use.");
  html << "</p><p>";
  html << _("ICC Examin's file selector is designed to work like a browser. Every new selected file is instantly shown in the aaplication for quick comparisions. When ICC Examin starts to parse files, it shows the yellow progressbar. Depending on the complexity of the file it may take some time to finish. The open gamut view takes extra time.");
  html << "</p><p>";
  html << _("VRML file are selectable alone. ICC Examin will not accept mixing with other files. The VRML gamut hull's from ArgyllCMS are the only known supported ones.");
  html << "</p><p>";
  html << _("ICC profiles can be loaded many for gamut comparisions. A named colour profile will only be accepted in the first position. Loading a CGATS measurement file after the belonging profile allowes to create the ICC Examin quality report. Some profiles, most of the standard CMYK ones, have the measurements already included.");
  html << "</p><p>";
  html << _("Multiple CGATS measurements can be loaded at once");
  html << "</p><p>";
  html << _("Files once loaded are observed by ICC Examin. The view will be actualised upon a modification.");
  html << "</p> \
<a name=\"Main_window\"></a><h3>";
  html << _("2.2 Main window"); html << "</h3> \
<p>";
  html << _("In normal mode ICC Examin shows first the tag list with the tag viewer below. Over the tag list resides the applications menubar.");
  html << "</p><p>";
  html << _("You can select an tag from the taglist and watch its content in the tag window below.");
  html << "</p><p>";
  html << _("On the bottom you see the status area of the window.");
  html << "</p> \
<a name=\"Complex_tags_navigation\"></a><h3>";
  html << _("2.3 Complex tags navigation"); html << "</h3> \
 \
<p>";
  html << _("Some tags contain complex data. For such tags may appear a choicebutton below the tag list. There you can select detailed views of a complex tags endities."); html << "</p> \
<a name=\"Windows_and_Views\"></a><h3>";
  html << _("2.4 Windows and Views"); html << "</h3> \
<a name=\"Gamut\"></a><h4>";
  html << _("2.4.1 Gamut"); html << "</h4> \
 \
<p>";
  html << _("The 3D gamut viewer can be selected from the menu bar Window &gt; Gamut. It shows the gamut of the device described by the profile with the absolute colorimetric rendering intent."); html << "</p><p>";
  html << _("The colour space shown is allways in CIE*Lab coordinates. "); html << "</p><p> \
";
  html << _("The Gamut Menus"); html << "</p>";
  html << _("File"); html << ": <ul><li>";
  html << _("Quit - allowes for exiting the application"); html << "</li></ul>";
  html << _("View"); html << ": <ul><li>";
  html << _("Full screen mode"); html << "</li></ul>";
  html << _("Settings"); html << ": <ul><li>";
  html << _("4 rendering intents + BPC"); html << "</li><li>";
  html << _("Native or separation Gamut. The later is the inbuild default. The native colour gamut can only be created when iccgamut from ArgyllCMS is available."); html << "</li></ul>";
  html << _("Windows"); html << ": \
<ul><li>";
  html << _("Show main application window, in case it is a icon.");
  html << "</li></ul> \
</p><a name=\"3D_navigation\"></a><h4>";
  html << _("2.4.2 3D navigation and keys"); html << "</h4> \
<p>";
  html << _("To navigate arround in a 3D view use the:");
  html << "</p> \
<ul><li>";
  html << _("left mouse button to rotate");
  html << "</li><li>";
  html << _("middle mouse button or Shift+left mouse button to move/clip");
  html << "</li><li>";
  html << _("right mouse button/ Strg+left mouse button to obtain additional menus. Among the additional menus are in:");
  html << "<ul><li>";
  html << _("top menu: the channel selection if appropriate");
  html << "<ul><li>";
  html << _("Slice plane submenu, with axis slicing in fly through style and rotation");
  html << "</li><li>";
  html << _("Illustration submenu, with the background colour selection and text/marker switching off and on");
  html << "</li></ul> \
</li></ul> \
</li></ul> \
<p>";
  html << _("From the context menu (ctrl+left mouse button or right mouse button) you can select the average human (CIE standard observer) visible spectral colours as a line."); html << "</p><p>";
  html << _("If meashurements are shown, the spot radius can be selected from the context menu. This is handy to easily detect dE's of above 2 or 4. The connection line goes from the measurement (white) to the deviation by the profile (red).");
  html << "</p><p>";
  html << _("Additional keys:");
  html << "</p><p>  <ul><li>";
  html << _("Gamut view:"); html << "</li> \
<ul> <li>";
  html << _("down key - reduce the slice plane distance"); html <<"</li><li>";
  html << _("up key - increase the slice plane distance"); html <<"</li><li>";
  html << _("home key - reset the slice plane distance"); html << "</li><li>";
  html << _("end key - set the slice plane distance to center"); html << "</li><li>";
  html << _("Colour spots can be enlarged or made smaller by pressing + or -.");
  html << "</li> \
</ul></li> \
</ul> \
<ul><li>";
  html << _("CLUT table line mode:"); html << "</li> \
<ul> \
     <li>";
  html << _(" number key - set the number of interpolation planes");
html << "</li> \
     <li>";
  html << _("'-' minus - reduce the actual interpolation plane"); html << "</li> \
     <li>";
  html << _("'+' plus - increase the actual interpolation plane"); html << "</li> \
     <li>";
  html << _("'/' slash - smaller steps for the minus and pus keys above"); html << "</li> \
     <li>";
  html << _("'*' star - larger steps for the minus and plus keys above"); html <<  "</li> \
     <li>";
  html << _("'_' underline - switch lines between one and three directions"); html << "</li> \
</ul></li> \
</ul> \
</p> \
<a name=\"Report_View\"></a><h4>";
  html << _("2.4.3 Report View"); html << "</h4> \
<p>";
  html << _("If the profile contains measurements a report is generated. You reach this by selecting from the top menu View-&gt;Report. These measurement data are compared to what is calculated with the profile by littleCMS - the internal used CMM.");
  html << "</p><p>";
  html << _("Such a comparision from device colours + measurements colours on the one side and device + profile/CMM colours on the other side are just one possible quality checking method.");
  html << "</p><p>";
  html << _("More comparision styles would be nice in giving better meaning for varying criteria. The differences are summarised in the reports top lines. Each like contains the the colour values and euclidian dE + dE2000. Two rectangles on the right side may help to see the colour differences visually. The first monitor profile is used for this.");
  html << "</p><p>";
  html << _("The report can be exported from the File menu in html format.");
  html << " \
</p> \
<a name=\"CGATS_view\"></a><h4>";
  html << _("2.4.4 CGATS view");
  html << "</h4> \
<p>";
  html << _("The measurement data is taken from the profile, if available, and transformed to fit in the CGATS standard. A window can be called from the View menu to show the corrected measurement output.");
 html << "</p> \
<a name=\"Grafic_card_gamma_tables\"></a><h4>";
  html << _("2.4.5 Grafic card gamma tables"); html << "</h4> \
<p>";
  html << _("The gamma tables in the grafic card are shown in an external window, callable from the View menu. It shows on osX/XFree86/Xorg the RGB calibration curves. Calibration data, as represented in this window, is traditionally not part of the colorimetrically bound ICC profiles. However, calibration data is influental to the correctness of the monitor ICC profiles. Profiling can happen without graphic card gamma tables and consequently the graphic card lookup tables should be linear in that case. For convenience, the monitor profile can be loaded from this window in the main profile viewer window for further examination. The grafic card is observed during the window being open. The window is sensible to positioning on multi monitor setups. The profiles equivalent is the vcgt tag. A loaded monitor profile should show the same calibration curves in the vcgt tag as the Grafic card gamma tables window or the Grafic card gamma tables window should be linear or not set.");
  html << "</p> \
<a name=\"Help\"></a><h4>";
  html << _("2.4.6 Help"); html << "</h4> \
<p>";
  html << _("The Help menus About button shows a short overview, license and other programers acknowledgement, who helped with providing theyre code for free."); html << "</p> \
<a name=\"Profile_elements\"></a><h2>";
  html << _("3 Profile elements"); html << "</h2> \
<a name=\"Header\"></a><h3>";
  html << _("3.1 Header"); html << "</h3> \
 \
<p>";
  html << _("Now you see the tag list has obtained some entries, showing the profiles tags. The first entry is allways the header. If the header is selected the tag window below shows many useful information about the profile. Among these information are:"); html << " \
</p> \
<ul><li>";
  html << _("the internal stored size of the profile"); html << "</li><li>";
  html << _("the prefered ColourMatchingModule (CMM)"); html << "</li><li>";
  html << _("ICC standard version"); html << "</li><li>";
  html << _("kind of usage"); html << "</li><li>";
  html << _("device colour space"); html << "</li><li>";
  html << _("the internal used reference colour space"); html << "</li><li>";
  html << _("date of creation"); html << "</li><li>";
  html << _("file type sinature (allways acsp)"); html << "</li><li>";
  html << _("and some more or less meaningfull informations");
  html << "</li></ul> \
<a name=\"Tags\"></a><h3>";
  html << _("3.2 Tag types"); html << "</h3> \
<p>";
  html << _("Below the header, which behaves like a fixed tag, are the variable tags listed. They appear in the same order as in the file. Starting with the tag number, then the tag signature and the type of content in the tag. The size of the tag follows and a small description gives you an idea af what the tag may be intended for.");
  html << "</p> \
<a name=\"text_.2F_desc_.2F_targ\"></a><h4>";
  html << _("3.2.1 text / desc / targ / mluc"); html << "</h4> \
 \
<p>";
  html << _("Tags of type text are the most simple ones. They include informations about the profile like License informations, Description for displaying on behalf of the whole profile, Measurement data , in CGATS text format and more.");
  html << "</p> \
<a name=\"curv\"></a><h4>";
  html << _("3.2.2 curv"); html << "</h4> \
<p>";
  html << _("An other type are curves. They are shown as such. Note the coordinates are all normalised to 0.0 -&gt; 1.0.");
  html << "</p> \
<a name=\"XYZ\"></a><h4>";
  html << _("3.2.3 XYZ"); html << "</h4> \
 \
<p>";
  html << _("XYZ tags show things like Mediawhitepoint or Primaries of monitors in the CIE*xy diagram. This diagram shows only the hue and intensity part of the CIE XYZ colour space. The Intensity of the CIE Y channel can be imagined as a view from top.");
  html << "</p><p>";
  html << _("Note: primaries and some curves are grouped to better understand their meaning in the profile.");
  html << "</p> \
<a name=\"mft1_.2F_mft2\"></a><h4>";
  html << _("3.2.4 mft1 / mft2"); html << "</h4> \
<p>";
  html << _("MFT1/2 tags are complex and contain a set of peaces to do a colour transformation. The choice button below the tag view names them. You can select the matrix, in-and output curves and the CLUT. This kind of tag is mostly independend of other tags. Only few tags like wtpt and chad may influence them.");
  html << "</p><p>";
  html << _("The CLUT view has various options to visualise the table. For instance you can make table appear coloured or select a channel of choice. The numbers in the status window are normalised to 0.0 -&gt; 1.0.");
  html << "</p><p>";
  html << _("They contain the result of hard work in researching the best translation of scattered measurements to equally spaced grid tables. Only this step makes colour transformation reasonably fast.");
  html << "</p><p>";
  html << _("The tag is described in in verson 2 of the ICC specification. In ICC specification version 4 a new even more complex tag is introduced and is calleed mAB.");
  html << "</p> \
<a name=\"sf32\"></a><h4>";
  html << _("3.2.5 sf32"); html << "</h4> \
 \
<p>";
  html << _("sf32 and similiar tags may contain only numbers. Theyre interpretation depends on the name of the tag. For instance the tag named chad is used to specify a colour transformation matrix, which was originally used to normalise the measured colours to D50 standard conditions.");
  html << "</p> \
<a name=\"ncl2\"></a><h4>";
  html << _("3.2.6 ncl2");
  html << "</h4> \
<p>";
  html << _("The ncl2 tag is used to store named colours in a list. This can be used to describe spot colours or describe a painters palette. Loading a profile containing such a tag into ICC Examin switches the 3D gamut view mode on. You see then the colours in 3D (CIE*Lab) and not as a list of numbers as in the tag viewer.");
  html << "</p> \
<a name=\"psid\"></a><h4>";
  html << _("3.2.7 psid"); html << "</h4> \
<p>";
  html << _(" The Profile identifier tag is used in device link profiles to identify the profiles used to build the device link. However there are other informations needed to repeat the creation of a device link profile, like rendering intents or K preservation flags and so on.");
  html << "</p> \
</p><p><br />";
  html << _("More tags can follow here...");
  html << "</p> \
<a name=\"Other_file_formats\"></a><h2>";
  html << _("4 Other file formats"); html << "</h2> \
 \
<a name=\"CGATS\"></a><h3>";
  html << _("4.1 Measurements CGATS"); html << "</h3> \
<p>";
  html << _("Measurement data in CGATS format can be opened in ICC Examin. In the file selector is a dedicated filter available. The data are shown as text and in the 3D view.");
  html << "</p> \
<a name=\"VRML\"></a><h3>";
  html << _("4.2 Model VRML"); html <<  "</h3> \
<p>";
  html << _("A subset of vrml alias wrl files is parsed. It allowes to open colour gamuts produced with argyll and saved with ICC Examin.");
  html << "</p><p>";
  html << _("It would be nice to allow as well lines to get parsed, which are used for visualisations by iccview for instance.");
  html << "</p>";
  html << "\n\n<br>\n</body></html>\n";

  DBG_PROG_ENDE
  return html.str();
}

std::string
getLizenzHtml()
{
  DBG_PROG_START
  std::stringstream html;
  html << getHtmlKopf( _("License") );

  html << " \
<p>\n";
  // HTML
  html << _(
"    ICC Examin is licensed under the GPL version 2."
"    Especially all icc_* files are licensed under the GPL.<br>"     
"    The other source files may contain devergine licenses, which are detectable from "
"file headers. ");
  html << " \
<br<br> \
</p>\n";

  html << " \
<HR> \
 \
 \
<H2><A NAME=\"SEC1\" HREF=\"http://www.gnu.org/licenses/gpl.html\">GNU GENERAL PUBLIC LICENSE</A></H2> \
<P> \
Version 2, June 1991 \
 \
</P> \
 \
Copyright (C) 1989, 1991 Free Software Foundation, Inc.   \
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA \
 \
Everyone is permitted to copy and distribute verbatim copies \
of this license document, but changing it is not allowed. \
 \
 \
 \
<H2><A NAME=\"SEC2\" HREF=\"http://www.gnu.org/licenses/gpl.html#TOC2\">Preamble</A></H2> \
 \
<P> \
  The licenses for most software are designed to take away your \
freedom to share and change it.  By contrast, the GNU General Public \
License is intended to guarantee your freedom to share and change free \
software--to make sure the software is free for all its users.  This \
General Public License applies to most of the Free Software \
Foundation's software and to any other program whose authors commit to \
using it.  (Some other Free Software Foundation software is covered by \
the GNU Library General Public License instead.)  You can apply it to \
your programs, too. \
 \
</P> \
<P> \
  When we speak of free software, we are referring to freedom, not \
price.  Our General Public Licenses are designed to make sure that you \
have the freedom to distribute copies of free software (and charge for \
this service if you wish), that you receive source code or can get it \
if you want it, that you can change the software or use pieces of it \
in new free programs; and that you know you can do these things. \
 \
</P> \
<P> \
  To protect your rights, we need to make restrictions that forbid \
anyone to deny you these rights or to ask you to surrender the rights. \
These restrictions translate to certain responsibilities for you if you \
distribute copies of the software, or if you modify it. \
 \
</P> \
<P> \
  For example, if you distribute copies of such a program, whether \
gratis or for a fee, you must give the recipients all the rights that \
you have.  You must make sure that they, too, receive or can get the \
source code.  And you must show them these terms so they know their \
rights. \
 \
</P> \
<P> \
  We protect your rights with two steps: (1) copyright the software, and \
(2) offer you this license which gives you legal permission to copy, \
distribute and/or modify the software. \
 \
</P> \
<P> \
  Also, for each author's protection and ours, we want to make certain \
that everyone understands that there is no warranty for this free \
software.  If the software is modified by someone else and passed on, we \
want its recipients to know that what they have is not the original, so \
that any problems introduced by others will not reflect on the original \
authors' reputations. \
 \
</P> \
<P> \
  Finally, any free program is threatened constantly by software \
patents.  We wish to avoid the danger that redistributors of a free \
program will individually obtain patent licenses, in effect making the \
program proprietary.  To prevent this, we have made it clear that any \
patent must be licensed for everyone's free use or not licensed at all. \
 \
</P> \
<P> \
  The precise terms and conditions for copying, distribution and \
modification follow. \
 \
</P> \
 \
 \
<H2><A NAME=\"SEC3\" HREF=\"gpl.html#TOC3\">TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION</A></H2> \
 \
 \
<P> \
 \
<STRONG>0.</STRONG> \
 This License applies to any program or other work which contains \
a notice placed by the copyright holder saying it may be distributed \
under the terms of this General Public License.  The \"Program\", below, \
refers to any such program or work, and a \"work based on the Program\" \
means either the Program or any derivative work under copyright law: \
that is to say, a work containing the Program or a portion of it, \
either verbatim or with modifications and/or translated into another \
language.  (Hereinafter, translation is included without limitation in \
the term \"modification\".)  Each licensee is addressed as \"you\". \
<P> \
 \
Activities other than copying, distribution and modification are not \
covered by this License; they are outside its scope.  The act of \
running the Program is not restricted, and the output from the Program \
is covered only if its contents constitute a work based on the \
Program (independent of having been made by running the Program). \
Whether that is true depends on what the Program does. \
 \
<P> \
 \
<STRONG>1.</STRONG> \
 You may copy and distribute verbatim copies of the Program's \
source code as you receive it, in any medium, provided that you \
conspicuously and appropriately publish on each copy an appropriate \
copyright notice and disclaimer of warranty; keep intact all the \
notices that refer to this License and to the absence of any warranty; \
and give any other recipients of the Program a copy of this License \
along with the Program. \
<P> \
 \
You may charge a fee for the physical act of transferring a copy, and \
you may at your option offer warranty protection in exchange for a fee. \
<P> \
 \
<STRONG>2.</STRONG> \
 You may modify your copy or copies of the Program or any portion \
of it, thus forming a work based on the Program, and copy and \
distribute such modifications or work under the terms of Section 1 \
above, provided that you also meet all of these conditions: \
<P> \
 \
<UL> \
 \
<LI><STRONG>a)</STRONG> \
     You must cause the modified files to carry prominent notices \
     stating that you changed the files and the date of any change. \
 \
<P> \
<LI><STRONG>b)</STRONG> \
     You must cause any work that you distribute or publish, that in \
     whole or in part contains or is derived from the Program or any \
     part thereof, to be licensed as a whole at no charge to all third \
     parties under the terms of this License. \
 \
<P> \
<LI><STRONG>c)</STRONG> \
     If the modified program normally reads commands interactively \
     when run, you must cause it, when started running for such \
     interactive use in the most ordinary way, to print or display an \
     announcement including an appropriate copyright notice and a \
     notice that there is no warranty (or else, saying that you provide \
     a warranty) and that users may redistribute the program under \
     these conditions, and telling the user how to view a copy of this \
     License.  (Exception: if the Program itself is interactive but \
     does not normally print such an announcement, your work based on \
     the Program is not required to print an announcement.) \
</UL> \
 \
These requirements apply to the modified work as a whole.  If \
identifiable sections of that work are not derived from the Program, \
and can be reasonably considered independent and separate works in \
themselves, then this License, and its terms, do not apply to those \
sections when you distribute them as separate works.  But when you \
distribute the same sections as part of a whole which is a work based \
on the Program, the distribution of the whole must be on the terms of \
this License, whose permissions for other licensees extend to the \
entire whole, and thus to each and every part regardless of who wrote it. \
<P> \
 \
Thus, it is not the intent of this section to claim rights or contest \
your rights to work written entirely by you; rather, the intent is to \
exercise the right to control the distribution of derivative or \
collective works based on the Program. \
<P> \
 \
In addition, mere aggregation of another work not based on the Program \
with the Program (or with a work based on the Program) on a volume of \
a storage or distribution medium does not bring the other work under \
the scope of this License. \
 \
<P> \
 \
<STRONG>3.</STRONG> \
 You may copy and distribute the Program (or a work based on it, \
under Section 2) in object code or executable form under the terms of \
Sections 1 and 2 above provided that you also do one of the following: \
 \
 \
<!-- we use this doubled UL to get the sub-sections indented, --> \
<!-- while making the bullets as unobvious as possible. --> \
<UL> \
 \
<LI><STRONG>a)</STRONG> \
     Accompany it with the complete corresponding machine-readable \
     source code, which must be distributed under the terms of Sections \
     1 and 2 above on a medium customarily used for software interchange; or, \
 \
<P> \
<LI><STRONG>b)</STRONG> \
     Accompany it with a written offer, valid for at least three \
     years, to give any third party, for a charge no more than your \
     cost of physically performing source distribution, a complete \
     machine-readable copy of the corresponding source code, to be \
     distributed under the terms of Sections 1 and 2 above on a medium \
     customarily used for software interchange; or, \
 \
<P> \
<LI><STRONG>c)</STRONG> \
     Accompany it with the information you received as to the offer \
     to distribute corresponding source code.  (This alternative is \
     allowed only for noncommercial distribution and only if you \
     received the program in object code or executable form with such \
     an offer, in accord with Subsection b above.) \
</UL> \
 \
The source code for a work means the preferred form of the work for \
making modifications to it.  For an executable work, complete source \
code means all the source code for all modules it contains, plus any \
associated interface definition files, plus the scripts used to \
control compilation and installation of the executable.  However, as a \
special exception, the source code distributed need not include \
anything that is normally distributed (in either source or binary \
form) with the major components (compiler, kernel, and so on) of the \
operating system on which the executable runs, unless that component \
itself accompanies the executable. \
<P> \
 \
If distribution of executable or object code is made by offering \
access to copy from a designated place, then offering equivalent \
access to copy the source code from the same place counts as \
distribution of the source code, even though third parties are not \
compelled to copy the source along with the object code. \
<P> \
 \
<STRONG>4.</STRONG> \
 You may not copy, modify, sublicense, or distribute the Program \
except as expressly provided under this License.  Any attempt \
otherwise to copy, modify, sublicense or distribute the Program is \
void, and will automatically terminate your rights under this License. \
However, parties who have received copies, or rights, from you under \
this License will not have their licenses terminated so long as such \
parties remain in full compliance. \
 \
<P> \
 \
<STRONG>5.</STRONG> \
 You are not required to accept this License, since you have not \
signed it.  However, nothing else grants you permission to modify or \
distribute the Program or its derivative works.  These actions are \
prohibited by law if you do not accept this License.  Therefore, by \
modifying or distributing the Program (or any work based on the \
Program), you indicate your acceptance of this License to do so, and \
all its terms and conditions for copying, distributing or modifying \
the Program or works based on it. \
 \
<P> \
 \
<STRONG>6.</STRONG> \
 Each time you redistribute the Program (or any work based on the \
Program), the recipient automatically receives a license from the \
original licensor to copy, distribute or modify the Program subject to \
these terms and conditions.  You may not impose any further \
restrictions on the recipients' exercise of the rights granted herein. \
You are not responsible for enforcing compliance by third parties to \
this License. \
 \
<P> \
 \
<STRONG>7.</STRONG> \
 If, as a consequence of a court judgment or allegation of patent \
infringement or for any other reason (not limited to patent issues), \
conditions are imposed on you (whether by court order, agreement or \
otherwise) that contradict the conditions of this License, they do not \
excuse you from the conditions of this License.  If you cannot \
distribute so as to satisfy simultaneously your obligations under this \
License and any other pertinent obligations, then as a consequence you \
may not distribute the Program at all.  For example, if a patent \
license would not permit royalty-free redistribution of the Program by \
all those who receive copies directly or indirectly through you, then \
the only way you could satisfy both it and this License would be to \
refrain entirely from distribution of the Program. \
<P> \
 \
If any portion of this section is held invalid or unenforceable under \
any particular circumstance, the balance of the section is intended to \
apply and the section as a whole is intended to apply in other \
circumstances. \
<P> \
 \
It is not the purpose of this section to induce you to infringe any \
patents or other property right claims or to contest validity of any \
such claims; this section has the sole purpose of protecting the \
integrity of the free software distribution system, which is \
implemented by public license practices.  Many people have made \
generous contributions to the wide range of software distributed \
through that system in reliance on consistent application of that \
system; it is up to the author/donor to decide if he or she is willing \
to distribute software through any other system and a licensee cannot \
impose that choice. \
<P> \
 \
This section is intended to make thoroughly clear what is believed to \
be a consequence of the rest of this License. \
 \
<P> \
 \
<STRONG>8.</STRONG> \
 If the distribution and/or use of the Program is restricted in \
certain countries either by patents or by copyrighted interfaces, the \
original copyright holder who places the Program under this License \
may add an explicit geographical distribution limitation excluding \
those countries, so that distribution is permitted only in or among \
countries not thus excluded.  In such case, this License incorporates \
the limitation as if written in the body of this License. \
 \
<P> \
 \
<STRONG>9.</STRONG> \
 The Free Software Foundation may publish revised and/or new versions \
of the General Public License from time to time.  Such new versions will \
be similar in spirit to the present version, but may differ in detail to \
address new problems or concerns. \
<P> \
 \
Each version is given a distinguishing version number.  If the Program \
specifies a version number of this License which applies to it and \"any \
later version\", you have the option of following the terms and conditions \
either of that version or of any later version published by the Free \
Software Foundation.  If the Program does not specify a version number of \
this License, you may choose any version ever published by the Free Software \
Foundation. \
 \
<P> \
 \
 \
<STRONG>10.</STRONG> \
 If you wish to incorporate parts of the Program into other free \
programs whose distribution conditions are different, write to the author \
to ask for permission.  For software which is copyrighted by the Free \
Software Foundation, write to the Free Software Foundation; we sometimes \
make exceptions for this.  Our decision will be guided by the two goals \
of preserving the free status of all derivatives of our free software and \
of promoting the sharing and reuse of software generally. \
 \
 \
 \
<P><STRONG>NO WARRANTY</STRONG></P> \
 \
<P> \
 \
<STRONG>11.</STRONG> \
 BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY \
FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN \
OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES \
PROVIDE THE PROGRAM \"AS IS\" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED \
OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF \
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS \
TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE \
PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING, \
REPAIR OR CORRECTION. \
 \
<P> \
 \
<STRONG>12.</STRONG> \
 IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING \
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR \
REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, \
INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING \
OUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED \
TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY \
YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER \
PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE \
POSSIBILITY OF SUCH DAMAGES. \
 \
<P> \
 \
 \
<H2>END OF TERMS AND CONDITIONS</H2> \
 \
 \
 \
";

  html << "\n\n<br>\n</body></html>\n";

  DBG_PROG_ENDE
  return html.str();
}

std::string
getDankHtml()
{
  DBG_PROG_START
  std::stringstream html;
  // HTML
  html << getHtmlKopf( _("Acknowledgement") );

  html << "\
<p>";
  // HTML
  html << _(
"My cordially thank is especially dedicated to Marti Maria and the many "
"writers of the Lcms-user email list.");
  html << " \
</p>\n \
<br> \
<br> \
    </font> \
    </font> \
    </td> \
  </tr> \
</table>\
<p>\n";
  // HTML
  html << _(
"ICC Examin uses following libraries:<br>\n");
  html << " \
<p>";
  // HTML
  html << _(
"  FLTK: Bill Spitzack and others www.fltk.org<br>\n");
#ifdef HAVE_FLU
  html << " \
</p>\n \
<p>";
  // HTML
  html << _(
"  FLU: Jason Bryan www.osc.edu/~jbryan/FLU/<br>\n");
#endif
  html << " \
</p>\n \
<p>";
  // HTML
  html << _(
"  littleCMS: Marti Maria www.littlecms.com<br>\n");
  html << " \
 <br> \
 <br> \
</p>\n \
<br>\n";
#ifdef HAVE_FTGL
  html << " \
<p>";
  // HTML
  html << _(
"  FTGL: Henry Maddocks homepages.paradise.net.nz/henryj/code/#FTGL<br>\n");
  html << " \
</p>";
#endif
  html << " \
<p>";
  // HTML
  html << _(
"integrated sources, lock at the according files for licence conditions:<br>\n");
  html << " \
<p>";
  // HTML
  html << _(
"  agv 3D Navigation: Philip Winston<br>\n");
  html << " \
</p>\n \
<p>";
  // HTML
  html << _(
"  bsp rendering: Norman Chin<br>\n");
  html << " \
</p>\n \
<br>\n";
  html << " \
</p>\n \
<p>";
  // HTML
  html << _(
"For the visualisation of gamut (external):<br>\n");
  html << " \
<p>";
  // HTML
  html << _( 
"  argyll: Greame Gill web.access.net.au/argyll/argyllcms.html<br<br><br>\n");
  html << " \
<br>\n";
  html << "\n\n<br>\n</body></html>\n";
  DBG_PROG_ENDE
  return html.str();
}

std::string
getLinksText()
{
  DBG_PROG_START
  std::stringstream text;
  // HTML
  text << _( "Project" ) <<"     :\n"<< "  http://www.oyranos.org/icc-examin/" << "\n";
  // HTML
  text << _( "Documentation" )<<":\n"<< "  http://www.oyranos.org/wiki/index.php?title=ICC_Examin";

  DBG_PROG_ENDE
  return text.str();
}

std::string
getInfoHtml()
{
  DBG_PROG_START
  //char HF[] = "#aaaaaa";  // emphasised
  std::stringstream html;

  // HTML
  html << getHtmlKopf( _("Configure Log") );

  std::string log = icc_config_log_;
  int n = icc_parser::suchenErsetzen( log, "\n", "<br>", 0 );
  n = icc_parser::suchenErsetzen( log, " ", "&nbsp;", 0 );
  n = icc_parser::suchenErsetzen( log, "################################################################", "", 0 );
  html <<
  "<br><h4>";
  html << _( "Configure Log" );
  html << "</h4>\n\
<p>\n" <<
  log;
  html << "\n\n<br>\n</body></html>\n";
  html << " \
</p>\n";

  if(n) n = 0; // fix compiler warning

  //DBG_NUM_S(html.str() )
  DBG_PROG_ENDE
  return html.str();
}


